package io.syndesis.common.util.immutable;

import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link SkipNullsTest.TestValue}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableTestValue.builder()}.
 */
@Generated(from = "SkipNullsTest.TestValue", generator = "Immutables")
@SuppressWarnings({"all"})
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
public final class ImmutableTestValue implements SkipNullsTest.TestValue {
  private final Set<String> noNulls;

  private ImmutableTestValue(Set<String> noNulls) {
    this.noNulls = noNulls;
  }

  /**
   * @return The value of the {@code noNulls} attribute
   */
  @Override
  public Set<String> noNulls() {
    return noNulls;
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SkipNullsTest.TestValue#noNulls() noNulls}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableTestValue withNoNulls(String... elements) {
    Set<String> newValue = createUnmodifiableSet(createSafeList(Arrays.asList(elements), false, true));
    return new ImmutableTestValue(newValue);
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link SkipNullsTest.TestValue#noNulls() noNulls}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of noNulls elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableTestValue withNoNulls(Iterable<String> elements) {
    if (this.noNulls == elements) return this;
    Set<String> newValue = createUnmodifiableSet(createSafeList(elements, false, true));
    return new ImmutableTestValue(newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableTestValue} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableTestValue
        && equalTo((ImmutableTestValue) another);
  }

  private boolean equalTo(ImmutableTestValue another) {
    return noNulls.equals(another.noNulls);
  }

  /**
   * Computes a hash code from attributes: {@code noNulls}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + noNulls.hashCode();
    return h;
  }


  /**
   * Prints the immutable value {@code TestValue} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "TestValue{"
        + "noNulls=" + noNulls
        + "}";
  }

  /**
   * Creates an immutable copy of a {@link SkipNullsTest.TestValue} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable TestValue instance
   */
  public static ImmutableTestValue copyOf(SkipNullsTest.TestValue instance) {
    if (instance instanceof ImmutableTestValue) {
      return (ImmutableTestValue) instance;
    }
    return ImmutableTestValue.builder()
        .from(instance)
        .build();
  }

  /**
   * Creates a builder for {@link ImmutableTestValue ImmutableTestValue}.
   * <pre>
   * ImmutableTestValue.builder()
   *    .addNoNulls|addAllNoNulls(String) // {@link SkipNullsTest.TestValue#noNulls() noNulls} elements
   *    .build();
   * </pre>
   * @return A new ImmutableTestValue builder
   */
  public static ImmutableTestValue.Builder builder() {
    return new ImmutableTestValue.Builder();
  }

  /**
   * Builds instances of type {@link ImmutableTestValue ImmutableTestValue}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "SkipNullsTest.TestValue", generator = "Immutables")
  @NotThreadSafe
  public static final class Builder {
    private List<String> noNulls = new ArrayList<String>();

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code TestValue} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder from(SkipNullsTest.TestValue instance) {
      Objects.requireNonNull(instance, "instance");
      addAllNoNulls(instance.noNulls());
      return this;
    }

    /**
     * Adds one element to {@link SkipNullsTest.TestValue#noNulls() noNulls} set.
     * @param element A noNulls element
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder addNoNulls(@Nullable String element) {
      if (element != null) this.noNulls.add(element);
      return this;
    }

    /**
     * Adds elements to {@link SkipNullsTest.TestValue#noNulls() noNulls} set.
     * @param elements An array of noNulls elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder addNoNulls(String... elements) {
      for (String element : elements) {
        if (element != null) this.noNulls.add(element);
      }
      return this;
    }


    /**
     * Sets or replaces all elements for {@link SkipNullsTest.TestValue#noNulls() noNulls} set.
     * @param elements An iterable of noNulls elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder noNulls(Iterable<String> elements) {
      this.noNulls.clear();
      return addAllNoNulls(elements);
    }

    /**
     * Adds elements to {@link SkipNullsTest.TestValue#noNulls() noNulls} set.
     * @param elements An iterable of noNulls elements
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder addAllNoNulls(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.noNulls.add(element);
      }
      return this;
    }

    /**
     * Builds a new {@link ImmutableTestValue ImmutableTestValue}.
     * @return An immutable instance of TestValue
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableTestValue build() {
      return new ImmutableTestValue(createUnmodifiableSet(noNulls));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<>();
    } else {
      list = new ArrayList<>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  /** Unmodifiable set constructed from list to avoid rehashing. */
  private static <T> Set<T> createUnmodifiableSet(List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptySet();
    case 1: return Collections.singleton(list.get(0));
    default:
      Set<T> set = new LinkedHashSet<>(list.size());
      set.addAll(list);
      return Collections.unmodifiableSet(set);
    }
  }
}
